// IR Temperature monitor
//
// Based on the Sparkfun IR Eval board (8 MHz Arduino 328P with Melaxis MLX90614 sensor)
// Drives a 128x96 pixel 3-wire OLED display (9-bit SPI).  Designed to operate off of
// a single cell LiION battery with constant 3.3V power supply.  Switch input for power
// control (through a p-channel mosfet switch that is turned on by both the power button
// and a digital output from the arduino).  Button input used to reset auto-off timer and
// to turn off.
//
// Arduino pin assignment
//   A0 : VSNS analog input (Vbatt / 2)
//   A1 : VREF analog input (Precision 2.5 volt external reference)
//   A2 : HOLD digital output (drive high to enable p-channel MOSFET to supply power)
//   A3 : nBUTTON digital input (low when button is depressed, high when button is unpressed)
//   D0 (TX): TX serial output (logs temperature during normal operation)
//   D1 (RX): RX serial input (unused)
//   D8 : STATUS LED (blinks during final minute of auto-off timer)
//   D11 : MOSI (9-bit serial data out to OLED)
//   D12 : nCS (Active low OLED chip-select)
//   D13 : SCK (serial clock out to OLED)
//
// Operation
//
// Temperature sense
//    1. Sampled 10 times per second
//    2. 4-entry averaging buffer
//
// Display
//    1. Shows temperature in both C and F (XX.X format)
//    2. Shows battery voltage (X.XX volts) [or alternating "Low" battery indication]
//    3. Shows battery voltage level (range 3.0 - 4.2 volts)
//    4. Shows auto-off countdown timer starting at 60 seconds
//
//    Drives an SSD1306 OLED controller in 3-wire mode (SCK, MOSI, nCS).  SSD1306 reset is connected
//    to the arduino reset.  A software (bit-banged) SPI interface is implemented in order to send
//    the 9-bit words needed in 3-wire mode (D/C is encoded as the upper bit).
//
// Power Control
//    1. Power is switched on when the button is pressed and enables the external p-channel MOSFET.
//       The button must be held until the code starts executing (after loader timeout).  This is
//       indicated by the OLED display being initialized.
//    2. Power is held by this code by asserting HOLD
//    3. An auto-off timer releases HOLD after 5 minutes unless the button is quickly pressed and
//       released (which sets another 5 minute interval)
//    4. Power may be immediately switched off by pressing the button for more than 2 seconds.
//    5. Auto-off for low battery
//
// Battery Management
//    1. Low battery warning starting at 3.4 volts
//    2. Auto-off at 3.0 volts
//
// Copyright (c) 2015 Dan Julio.  All rights reserved.
//
// Licensed under the Creative Commons Attribution License.  Just let them know
// where you got the code.
//
// OLED display and fonts obtained from 43oh (www.43oh.com) for the first generation OLED display
// that was used in this project.  Thank you!
//

    
// ==============
// Globals from other modules that must be included at the top of the sketch to compile (jenky arduino environment...)
// ==============

//
// Timer related
//
#define TIMER_INIT (25 * 1000)
boolean timerTick;
boolean timerSecTick;


//
// OLED related
//
enum e_columnIncrementFlag { NO_FONT, SMALL_FONT, LARGE_FONT };
void OLED_Printf(char *format, ...);


void setup() {
  InitMisc();   // Done first to set power enable output (in case user releases button before the rest of the init is done)
  SSD1306Init();
  InitTimer();
  InitBattery();
  InitButton();
  InitTemp();
}

void loop() {
  EvalTimer();  // Done first to get tick values for this eval
  EvalButton(); // Done second to get current button state for this eval
  EvalBattery();
  EvalMisc();
  EvalTemp();
}

